/*
 * Copyright (c) 2020-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * BSD-3-Clause
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef SRC_DCMD_LINUX_COMPCHANNEL_H_
#define SRC_DCMD_LINUX_COMPCHANNEL_H_

namespace dcmd {

struct compchannel_ctx {
    LPOVERLAPPED overlapped;
    uint32_t eqe_nums;
};

class compchannel {
private:
    ctx_handle m_ctx;
    cq_handle m_cq_obj;
    comp_channel m_event_channel;
    bool m_binded;
    bool m_solicited;

public:
    compchannel()
    {
        m_ctx = nullptr;
        m_binded = false;
    }
    compchannel(ctx_handle handle);
    virtual ~compchannel();

    int bind(cq_handle cq_obj, bool solicited_only);
    int unbind();
    int get_comp_channel(event_channel*& ch);
    int request(compchannel_ctx& cc_ctx);
    int query(void*& cq_ctx);
    void flush(uint32_t n_events);
};

} /* namespace dcmd */

#endif /* SRC_DCMD_LINUX_COMPCHANNEL_H_ */
